/*******************************************************************************
 * Copyright (c) 2013, SWITCH
 ******************************************************************************/
package ch.SWITCH.uniapp;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.util.Log;

import com.google.gson.internal.StringMap;

/**
 * @author latzer
 * 
 *         SharedPreferences wrapper for storing AAI Attributes, providing an
 *         AsyncTask for attribute refresh
 */
public class AAIAttributeStorage {
	/**
	 * Name for the shared preferences
	 */
	private static final String AAI_ATTRIBUTES = "aai_attributes";

	private static final String LOG_TAG = "AttributeStorage";

	/**
	 * The character for separating array values in the shared preferences
	 */
	private static final String ARRAY_SEPARATOR = ";";

	private SharedPreferences preferences;
	private Context context;

	/**
	 * Standard constructor
	 * 
	 * @param context
	 *            app context
	 */
	public AAIAttributeStorage(Context context) {
		this.context = context;
		preferences = context.getSharedPreferences(AAI_ATTRIBUTES, Context.MODE_PRIVATE);
	}

	/**
	 * Retrieve an attribute. If this attribute is an array, the return value
	 * contains all elements concatenated by ";"
	 * 
	 * @param key
	 *            the key of the attribute
	 * @return the attribute(s) as a String, or null if it wasn't found
	 */
	public String getAttribute(String key) {
		return preferences.getString(key, null);
	}

	/**
	 * @param key
	 *            the key of the attribute
	 * @return the attribute(s) split into an array
	 */
	public String[] getAttributeArray(String key) {
		return getAttribute(key).split(ARRAY_SEPARATOR);
	}

	/**
	 * Calls {@link SharedPreferences#getAll()} on the underlying Preferences
	 * instance
	 * 
	 * @return all values from the preferences
	 */
	public Map<String, ?> getAll() {
		return preferences.getAll();
	}

	/**
	 * @author latzer
	 * 
	 *         Downloads the AAI Attributes for the supplied access_key and
	 *         stores them in shared preferences.
	 * 
	 */
	public class UpdateAttributesTask extends ApiCallTask {
		private String accessToken;

		public UpdateAttributesTask(String accessToken) {
			this.accessToken = accessToken;

		}

		/**
		 * Build the URL to request the attributes
		 * 
		 * @return
		 */
		protected String buildRequestUrl() {
			return Util.getApiUriBuilder(context)
					.appendPath("attributes")
					.appendQueryParameter("access_token", accessToken)
					.build()
					.toString();
		}

		/**
		 * loop through all the attributes and write them to sharedPreferences
		 * 
		 * @param data
		 */
		protected void storeAttributes(StringMap data) {
			Editor editor = preferences.edit();

			editor.clear(); //remove old attributes

			for (Iterator keys = data.keySet()
					.iterator(); keys.hasNext();) {
				String key = (String) keys.next();
				Object value = data.get(key);
				String attribute;

				if (value instanceof String) {
					attribute = (String) value;
				} else if (value instanceof List) {
					List list = (List) value;
					StringBuffer buf = new StringBuffer();

					for (Iterator listItems = list.iterator(); listItems.hasNext();) {
						buf.append((String) listItems.next());
						if (listItems.hasNext()) {
							buf.append(ARRAY_SEPARATOR);
						}
					}
					attribute = buf.toString();
				} else {
					Log.e(LOG_TAG, "Could not process key '" + key + "', its value is an instance of "
							+ value.getClass()
							.getName());
					attribute = null;
				}
				Log.d(LOG_TAG, "Storing " + key + ":" + attribute);
				editor.putString(key, attribute);
			}
			editor.commit();

		}

		@Override
		protected JSend doInBackground(String... params) {
			String requestUrl = buildRequestUrl();
			Log.d(LOG_TAG, "RequestURL: " + requestUrl);

			JSend response = super.apiCallJSend(requestUrl);

			if (response.getStatus()
					.equals(JSend.SUCCESS)) {
				storeAttributes((StringMap) response.getData());
			} else {
				Log.e(LOG_TAG, "Got an unsuccessful response");
			}
			return response;
		}
	}

}
