/*******************************************************************************
 * Copyright (c) 2013, SWITCH
 ******************************************************************************/
package ch.SWITCH.uniapp;

import java.util.Calendar;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.util.Log;

public abstract class BaseLoginActivity extends Activity {
	/**
	 * The key of the access_token for sharedPrefs
	 */
	protected static final String KEY_ACCESSTOKEN = "accesstoken";
	/**
	 * The key of the access_token expiry for sharedPrefs
	 */
	protected static final String KEY_ACCESSEXPIRES = "accessexpires";

	private static final String LOG_TAG = "BaseLoginActivity";

	/**
	 * returns the current access token, or null if it is not set
	 * 
	 * @param ctx
	 *            application context
	 * @throws TokenExpiredException
	 *             if the token expired
	 * @return the current access token, or null if it is expired or not set
	 */
	public static String getAccessToken(Context ctx) throws TokenExpiredException {
		SharedPreferences prefs = ctx.getSharedPreferences(AAILoginActivity.class.toString(), MODE_PRIVATE);
		String token = prefs.getString(KEY_ACCESSTOKEN, null);
		if (token == null) {
			return null;
		}
		int expiry = prefs.getInt(KEY_ACCESSEXPIRES, 0);
		if (isExpired(expiry)) {
			Log.d(LOG_TAG, "Access token expired");
			throw new TokenExpiredException();
		}
		return token;
	}

	/**
	 * Determines if the given unix timestamp is already in the past
	 * 
	 * @param utime
	 *            a unix timestamp
	 * @return true if it's in the past
	 */
	private static boolean isExpired(int utime) {
		Calendar expiry = Calendar.getInstance();
		expiry.setTimeInMillis(((long) utime) * 1000);
		Calendar now = Calendar.getInstance();
		return expiry.before(now);

	}

	/**
	 * Sets the access token and expiry date
	 * 
	 * @param ctx
	 *            app context
	 * @param token
	 *            access token
	 * @param expires
	 *            expiry unixtime
	 */
	public static void setAccessToken(Context ctx, String token, int expires) {
		Editor editor = ctx.getSharedPreferences(AAILoginActivity.class.toString(), MODE_PRIVATE)
				.edit();
		editor.putString(KEY_ACCESSTOKEN, token);
		editor.putInt(KEY_ACCESSEXPIRES, expires);
		editor.commit();
	}

	/**
	 * Reads the home activity from the configuration
	 * 
	 * @return the home activity class
	 */
	protected Class getHomeActivity() {
		String className = getString(R.string.uniapp_homeActivity);
		try {
			return Class.forName(className);
		} catch (ClassNotFoundException e) {
			throw new RuntimeException("Home activity " + className + " not found", e);
		}
	}

	/**
	 * Handles JSend error objects
	 * 
	 * @param response
	 */
	protected void handleError(JSend response) {
		int sc = response.getCode();
		if (sc == StatusCode.ERROR_TOKEN_EXPIRED) {
			setAccessToken(this, null, 0);
			showErrorDialog("Your login has expired. Please log in again.", true);
			return;
		}
		showErrorDialog(response.getMessage(), false);

	}

	/**
	 * Queries the user's attributes and starts the home activity on success
	 * 
	 */
	protected class QueryAndLauchHomeTask extends AAIAttributeStorage.UpdateAttributesTask {

		private String accessToken;
		protected ProgressDialog pd;

		/**
		 * @param storage
		 *            an instance of AAIAttributeStorage for storing the
		 *            retrieved attributes
		 * @param accessToken
		 *            the access token to authenticate with
		 */
		public QueryAndLauchHomeTask(AAIAttributeStorage storage, String accessToken) {
			storage.super(accessToken);
			this.accessToken = accessToken;
		}

		@Override
		protected void onPreExecute() {
			Log.d(LOG_TAG, "Querying attributes for accessToken  '" + accessToken + "'");
			pd = ProgressDialog.show(BaseLoginActivity.this, "", "Refreshing Attributes...", true);
			super.onPreExecute();
		}

		@Override
		protected void onPostExecute(JSend response) {
			pd.dismiss();
			if (response.getStatus()
					.equals(JSend.SUCCESS)) {
				Intent intent = new Intent(getApplicationContext(), getHomeActivity());
				intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
				intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
				startActivity(intent);
				finish(); // we don't want the loginActivity to appear
			} else {
				this.onError(response); // if any errors occur, handle this
				// specifically
			}
			super.onPostExecute(response);
		}

		/**
		 * Manages displaying the error and possibly takes further actions
		 * 
		 * @param response
		 *            the JSend response
		 */
		protected void onError(JSend response) {
			// a login error might be fixed by logging in again...
			showErrorDialog("An error occured while logging you in, please try again. (" + response.getMessage() + ")",
					true);
			setAccessToken(BaseLoginActivity.this, null, 0);
		}

	}

	/**
	 * Shows an error dialog box
	 * 
	 * @param message
	 *            the message to display
	 * @param finish
	 *            whether to finish the activity after the dialog is dismissed
	 * 
	 */
	protected void showErrorDialog(String message, final boolean finish) {
		final AlertDialog alertDialog = new AlertDialog.Builder(BaseLoginActivity.this).create();
		alertDialog.setTitle("Error");
		Log.d(LOG_TAG, "Displaying error dialog with message: " + message);
		Thread.dumpStack();
		if (message != null || "".equals(message)) {
			alertDialog.setMessage(message);
		} else {
			alertDialog.setMessage("Unknown error");
		}
		alertDialog.setButton(AlertDialog.BUTTON_NEUTRAL, "Ok", new OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				alertDialog.dismiss();
				if (finish)
					finish();
			}
		});
		alertDialog.show();

	}

}
