<?php
/* Copyright (c) 2013, SWITCH */

// Initialize session, constants, database
require_once 'config.php';

// Include the storage models
require_once 'lib/ClientModel.php';
require_once 'lib/ScopeModel.php';
require_once 'lib/SessionModel.php';
require_once 'lib/Implicit.php';
require_once 'lib/AuthCodeManual.php';
require_once 'lib/MobileProxyAuthServer.php';
require_once 'lib/JSend.php';
require_once 'lib/Util.php';

// Initiate the Request handler
$request = new \OAuth2\Util\Request();

$sessionModel = new SessionModel;
$scopeModel = new ScopeModel;

// Initiate the auth server with the models
$authServer = new MobileProxyAuthServer(new ClientModel, $sessionModel, $scopeModel);
$authServer->setExpiresIn(TOKEN_TTL);

// Initiate the resource server with the models
$resourceServer = new \OAuth2\ResourceServer($sessionModel, $scopeModel);

// Enable support for the authorization code grant
$authServer->addGrantType(new Implicit($authServer));
$authServer->addGrantType(new AuthCodeManual($authServer));

$app->error($jsonErrorHandler);

//function for checking whether the request supplied a basic auth header authenticating them as a client
$checkClient = function() use ($app, $authServer){
	return function() use($app, $authServer){
		try{
			if(!$authServer->getStorage('client')->validateClientAuth()){
				$res = $app->response();
				$jsend = new JSend;
				$jsend->setStatus("fail");
				$jsend->setMessage("Client authentication is not valid.");
				$res->body($jsend);
				$app->stop();
			}
		} catch (ClientValidationFailedException $e){
			//auth header is missing
			$res = $app->response();
			$jsend = new JSend;
			$jsend->setStatus("fail");
			$jsend->setMessage($e->getMessage());
			$res->body($jsend);
			$app->stop();
		}
	};
};

// Clients will redirect to this address
$app->get('/signin/:app', function ($appName) use ($authServer, $app) {

	// Get device name for recognition in the admin panel and logs
	$device = $app->request()->params('device');
	$device = $device ? $device : "unknown";

	// Tell the auth server to check the required parameters are in the query string
	try{
		$params = $authServer->checkAuthoriseParams();
		$params['device'] = $device;
		$app->getLog()->info("Signin request with device name '$device'");
	} catch (Exception $e){
		$app->getLog()->info("Invalid Signin request with device name '$device': ". $e->getMessage());
		throw $e;
	}

	// Save the verified parameters to the user's session
	$_SESSION['params'] = serialize($params);

	// Redirect the user to sign-in
	$app->redirect(API_URL."aailogin/$appName");

});

//build a request for an alternate login
$app->get('/alt/:app', function($appName) use($app, $authServer){

	//get the id for the called app to initiate a login
	$clientId = $authServer->getStorage('client')->getClientId($appName);
	if(!$clientId) {
		$res = $app->response();
		$jsend = new JSend;
		$jsend->setStatus('error');
		$jsend->setMessage("App '$appName' not found");
		$res->body($jsend);
		$app->stop(404);
	}

	$uri = OAuth2\Util\RedirectUri::make(API_URL."signin/$appName", array(
		'client_id' => $clientId,
		'redirect_uri' => 'none',
		'response_type' => 'code_manual',
		'scope' => 'all',
		'device' => 'ALT_LOGIN'
	));
	$app->redirect($uri);
});

// AAI-Protected URL, triggers an AAI login
$app->get('/aailogin/:app', function($appName) use($app, $authServer){
	/* $appName is unused here, but required by the webserver for differentiating
	 * between shibboleth resources */

	//verify that we have an AAI session
	$uniqueID = @$_SERVER['uniqueID'];
	if( !$uniqueID ) {
		throw new Exception('No uniqueID in header - make sure this page is AAI protected');
	}

	$persistentID = @$_SERVER['persistent-id'];
	if( !$persistentID ) {
		throw new Exception('No persistent-id in header. Make sure your your identity provider sends this header.');
	}

	if ( ! isset($_SESSION['params'])) {
		throw new Exception('Missing auth parameters');
	}

	$params = unserialize($_SESSION['params']);

	// user and persistent id are required to create the session
	$params['user_id'] = $uniqueID;
	$params['persistent-id'] = $persistentID;

	/*
	 * usually here would be an authorization step - this is already handled over the AAI login.
	 */

	switch($params['response_type']){
	case "code_manual": //alternate login, display auth code in browser
		$_SESSION['params'] = serialize($params);
		$app->redirect(API_URL."devicename");

	case "token": //mobile login, redirect to app with the access token as parameter
		$params['grant_type'] = 'implicit';
		$params['token'] = $authServer->issueAccessToken($params);

		//send the user back to the app
		$uri = OAuth2\Util\RedirectUri::make($params['redirect_uri'], array(
			"access_token" => $params['token']['access_token'],
			"expires" => $params['token']['expires']
		));

		$app->getLog()->info("Implicit login complete, access_token: " . Util::truncateToken($params['token']['access_token']));

		/*
		 * using a Refresh header rather than Location so browsers which don't support the uniapp scheme
		 * show a HTML page instead of doing nothing
		 * (at the time this was coded both firefox and chrome did not do anything with a Location header
		 * to an uniapp:// url - not even an error message!)
		 */
		$response = $app->response();
		$response->body('Redirecting you to your app <a href="'.$uri.'">here</a>');
		$response['Refresh'] = "0; $uri";
		$app->stop();
	}

});

//Requests a name for the device for later identification in the administration view
$app->get('/devicename', function() use ($app){

	$app->render('devicename.tpl', array(
		'resourceRoot' => RESOURCE_ROOT,
		'title' => 'Mobile Proxy Login',
		'h1' => 'Device Name'
	));

});

//retrieves the device name and redirects to the auth code
$app->post('/devicename', function() use ($app, $authServer){

	$params = unserialize($_SESSION['params']);

	$params['device'] = $app->request()->params('device');
	$authCode = $authServer->newAuthoriseRequest('user',$params['user_id'], $params);

	$_SESSION['params'] = serialize($params);

	$app->getLog()->info("Device name entered: '" .$params['device']."'");
	$app->getLog()->info("Authorization code issued: " . Util::truncateToken($authCode));

	$app->redirect(API_URL."display_code/$authCode");

});

//display an authorization code
$app->get('/display_code/:code', function($code) use ($app, $htmlErrorHandler){
	$app->error($htmlErrorHandler);

	$app->render('displayCode.tpl', array(
		'resourceRoot' => RESOURCE_ROOT,
		'title' => 'Mobile Proxy Login',
		'h1' => 'Authorization code',
		'code' => $code,
		'qrUrl' => URL_QR_GENERATOR
	));

});

// The exchange the authorization code for an access token
$app->post('/access_token', function() use ($authServer, $app) {

	header('Content-type: application/json');

	try {
		// Issue an access token
		$p = $authServer->issueAccessToken();
		echo json_encode($p);

	} catch (Exception $e) {

		// Show an error message
		// TODO JSend this
		echo json_encode(array('error' => $e->getMessage(), 'error_code' => $e->getCode()));
	}

	$app->getLog()->info("Authorization code exchanged for access token: " . Util::truncateToken($p['access_token']));

});

//validates the access token, returning the corresponding client data
$app->get('/validate', $checkClient(), function() use ($app, $authServer){

	$token = $app->request()->params('access_token');
	if(!$token){
		$app->getLog()->warn('malformed access token validation: no access_token parameter');
		throw new Exception('access_token is missing');
	}

	$validation = $authServer->getStorage('session')->validateAccessToken($token);

	$res = $app->response();
	$jsend = new JSend;
	if($validation){
		$app->getLog()->info('successfully validated  access token: '. Util::truncateToken($token));
		$jsend->setStatus("success")
			->setData($validation);
	} else {
		$app->getLog()->warn('access token validation failed: '. Util::truncateToken($token));
		$jsend->setStatus("error")
			->setMessage("access_token '$token' invalid");
	}
	$res->body($jsend);

});

//revokes the given access token
$app->get('/revoke', function() use ($app, $authServer){

	$token = $app->request()->params('access_token');
	if(!$token) {
		$app->getLog()->warn('malformed access token revocation: no access_token parameter');
		throw new Exception('access_token is missing');
	}

	$app->getLog()->info('revoking access token: '.Util::truncateToken($token));

	$authServer->getStorage('session')->revokeAccessToken($token);

	$res = $app->response();

	$jsend = new JSend;
	$jsend->setStatus("success");

	$res->body($jsend);

});

/*
 * Function called when each endpoint is called to check the access token
 * exists and is valid
 */
$checkToken = function () use ($resourceServer) {

	return function() use ($resourceServer) {
		// Test for token existance and validity
			try {
				$resourceServer->isValid();
			}

			// The access token is missing or invalid...
			catch (\OAuth2\Exception\InvalidAccessTokenException $e) {
				$app = \Slim\Slim::getInstance();
				$res = $app->response();
				$res['Content-Type'] = 'application/json';
				$res->status(403);
				$jsend = new JSend;
				$jsend->setStatus("error");
				$jsend->setMessage($e->getMessage());
				$res->body($jsend);
				$app->stop();
			}
		};

};

// Retrieve the AAI attributes for provided access token
$app->get('/attributes', $checkToken(), function() use ($resourceServer, $app, $sessionModel, &$valid) {

	$token = $app->request()->params('access_token');
	if(!$token) {
		$app->getLog()->warn('malformed attribute query: no access_token parameter');
		throw new Exception('access_token is missing');
	}

	$res = $app->response();
	$res['Content-Type'] = 'application/json';

	//run the attribute query
	$jsend = new JSend;
	try{
		$app->getLog()->info('queried attributes for token: '.Util::truncateToken($token));
		$attributes = Util::attributeQuery($token, $sessionModel);
		$jsend->setStatus("success")->setData($attributes);
		$res->body($jsend);
	} catch ( Exception $e){
		$app->getLog()->warn('Error querying attributes for token '.Util::truncateToken($token).': ' . $e->getMessage());
		throw new Exception('Error while querying your attributes.');
	}

});

$app->run();
